#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

# DO NOT REMOVE! __all__ is used to locate frozen definitions
__all__ = (
    "ugh_", "ant_", "bitm", "trak", "snd_", "shad", "hsc_", "pphy",
    #"adlg", "mdlg", "bipd", "bsdt", "____", "devo", "whip", "char", "gldf",
    #"clwd", "coll", "coln", "colo", "cont", "bloc", "crea", "jpt_", "deca",
    #"DECR", "DECP", "dobc", "devi", "ctrl", "lifi", "mach", "udlg", "effe",
    #"eqip", "garb", "matg", "grhi", "hudg", "hmt_", "hud_", "item", "itmc",
    #"lens", "ligh", "MGS2", "tdtl", "foot", "mpdt", "metr", "hlmt", "jmad",
    #"mcsr", "unic", "mulg", "mply", "goof", "nhdt", "obje", "prt3", "PRTM",
    #"pmov", "fpch", "phys", "phmo", "pixl", "fog ", "proj", "mode", "sbsp",
    #"ltmp", "scnr", "ai__", "_ipd", "cin_", "clu_", "____", "_rea", "dec_",
    #"dc_s", "dgr_", "_qip", "_cen", "_sce", "sslt", "_igh", "trg_", "_ehi",
    #"_eap", "scen", "egor", "stem", "slit", "spas", "sky_", "sncl", "spk_",
    #"sfx_", "snde", "lsnd", "snmx", "ssce", "BooM", "styl", "sily", "unit",
    #"unhi", "wgtz", "skin", "wgit", "wigl", "vehi", "vehc", "vrtx", "weap",
    #"_fx_", "wphi", "weat", "wind"
    )
