#
# This file is part of Reclaimer.
#
# For authors and copyright check AUTHORS.TXT
#
# Reclaimer is free software under the GNU General Public License v3.0.
# See LICENSE for more information.
#

# DO NOT REMOVE! __all__ is used to locate frozen definitions
__all__ = ("composer_playlist", "pc_gametype", "xbox_gametype")
